*******************************************************************
* PROGRAMME: COST FUNCTION VARIABLE PREPARATION
*******************************************************************
*** IMPORT DATA
clear
set more off
* cd to put in relevant directory here
use "Bank_macro_data.dta"

*******************************************************************
*** GENERATING TOTAL COST
* Cost = total cost, including interest cost, of the production of financial services (so excludes non-operating expenses)
gen total_cost = int_exp_total + comp_benefits

*******************************************************************
*** GENERATING OUTPUTS OF BANK PRODUCTION
* y = (cash_equiv, total_invest, net_loans, invest_fhlb, other_assets_bnk, other_nonloan_assets)
* I use the quantity of output here, based on assets. Other papers use income as a measure.
gen y_1 = cash_equiv
gen y_2 = total_invest
gen y_3 = net_loans
gen y_4 = other_assets_bnk
gen y_5 = non_int_inc - serv_charge_deposits

*******************************************************************
*** GENERATING INPUT PRICES OF BANK PRODUCTION
* w_p = unit input price for labour and physical assets, measured by the average cost of non-financial inputs to banks
gen w_p = comp_benefits/total_assets
* w_d = unit input price for deposits
gen w_d = int_exp_total/(total_deposits + st_debt_bnk + lt_debt_bnk)

*******************************************************************
*** GENERATING NETPUTS
* k = financial equity capital
gen k = total_equity - allow_ll
* z = physical capital
gen p = nppe
* n = nonperforming assets or nonperforming loans
gen n = non_performing_loans

*******************************************************************
*** GENERATE INPUT SHARES
gen S_d = int_exp_total/total_cost 
gen S_p = comp_benefits/total_cost

*******************************************************************
*** GENERATING LOG VARIABLES
gen ln_total_assets = ln(total_assets)
foreach var of varlist total_cost-n {
generate ln_`var' = ln(`var')
}

*******************************************************************
*** GENERATING THE VARIABLES FOR THE COST FUNCTION ESTIMATION

* LINEAR HOMOGENEITY CONSTRAINTS, dividing total cost and w_p by w_d
gen cons_ln_w = ln_w_p - ln_w_d
gen cons_ln_total_cost = ln_total_cost - ln_w_d

* ln_y_i*ln_y_i TERMS
gen H_ln_y_1_ln_y_1  = 0.5*ln_y_1*ln_y_1 
gen H_ln_y_2_ln_y_2  = 0.5*ln_y_2*ln_y_2 
gen H_ln_y_3_ln_y_3  = 0.5*ln_y_3*ln_y_3
gen H_ln_y_4_ln_y_4  = 0.5*ln_y_4*ln_y_4 
gen H_ln_y_5_ln_y_5  = 0.5*ln_y_5*ln_y_5 

* ln_y_i*ln_y_j TERMS WITH SYMMETRY CONSTRAINTS, such that a(i,j) = a(j,i)
gen cons_H_ln_y_1_ln_y_2 = ln_y_1*ln_y_2 
gen cons_H_ln_y_1_ln_y_3 = ln_y_1*ln_y_3 
gen cons_H_ln_y_1_ln_y_4 = ln_y_1*ln_y_4 
gen cons_H_ln_y_1_ln_y_5 = ln_y_1*ln_y_5 
gen cons_H_ln_y_2_ln_y_3 = ln_y_2*ln_y_3
gen cons_H_ln_y_2_ln_y_4 = ln_y_2*ln_y_4
gen cons_H_ln_y_2_ln_y_5 = ln_y_2*ln_y_5
gen cons_H_ln_y_3_ln_y_4 = ln_y_3*ln_y_4
gen cons_H_ln_y_3_ln_y_5 = ln_y_3*ln_y_5
gen cons_H_ln_y_4_ln_y_5 = ln_y_4*ln_y_5

* ln_w_i*ln_w_j TERMS WITH LINEAR HOMOGENEITY CONSTRAINTS, i.e. dividing w_p by w_d
gen cons_ln_w_ln_w = 0.5*cons_ln_w*cons_ln_w

* ln_k*ln_k TERMS
gen H_ln_k_ln_k = 0.5*ln_k*ln_k 

* ln_y_i*ln_w TERMS WITH LINEAR HOMOGENEITY CONSTRAINTS, i.e. dividing w_p by w_d
gen cons_y_1_ln_w = ln_y_1*cons_ln_w
gen cons_y_2_ln_w = ln_y_2*cons_ln_w
gen cons_y_3_ln_w = ln_y_3*cons_ln_w
gen cons_y_4_ln_w = ln_y_4*cons_ln_w
gen cons_y_5_ln_w = ln_y_5*cons_ln_w

* ln_y_i*ln_k TERMS
gen ln_y_1_ln_k    = ln_y_1*ln_k 
gen ln_y_2_ln_k    = ln_y_2*ln_k 
gen ln_y_3_ln_k    = ln_y_3*ln_k 
gen ln_y_4_ln_k    = ln_y_4*ln_k 
gen ln_y_5_ln_k    = ln_y_5*ln_k 

* ln_w*ln_k TERMS WITH LINEAR HOMOGENEITY CONSTRAINTS, i.e. dividing w_p by w_d
gen cons_ln_w_ln_k = ln_k*cons_ln_w 

* ln_n*ln_n TERMS
gen H_ln_n_ln_n      = 0.5*ln_n*ln_n 

* ln_y_i*ln_n TERMS
gen ln_y_1_ln_n    = ln_y_1*ln_n 
gen ln_y_2_ln_n    = ln_y_2*ln_n 
gen ln_y_3_ln_n    = ln_y_3*ln_n 
gen ln_y_4_ln_n    = ln_y_4*ln_n 
gen ln_y_5_ln_n    = ln_y_5*ln_n 

* ln_w*ln_n TERMS WITH LINEAR HOMOGENEITY CONSTRAINTS, i.e. dividing w_p by w_d
gen cons_ln_w_ln_n = ln_n*cons_ln_w

* ln_k*ln_n TERMS
gen ln_k_ln_n	= ln_n*ln_k 

* ln_n*ln_n TERMS
gen H_ln_p_ln_p      = 0.5*ln_p*ln_p 

* ln_y_i*ln_n TERMS
gen ln_y_1_ln_p    = ln_y_1*ln_p 
gen ln_y_2_ln_p    = ln_y_2*ln_p 
gen ln_y_3_ln_p    = ln_y_3*ln_p 
gen ln_y_4_ln_p    = ln_y_4*ln_p 
gen ln_y_5_ln_p    = ln_y_5*ln_p 

* ln_w*ln_n TERMS WITH LINEAR HOMOGENEITY CONSTRAINTS, i.e. dividing w_p by w_d
gen cons_ln_w_ln_p = ln_p*cons_ln_w

* ln_k*ln_p TERMS
gen ln_k_ln_p	= ln_p*ln_k 

* ln_p*ln_n TERMS
gen ln_p_ln_n	= ln_p*ln_n

*******************************************************************
*** GENERATING THE TRIGONOMETRIC TERMS

sort company date 
forvalues i = 1(1)5 {
	egen a`i' = min(ln_y_`i') if date > 2000
	egen b`i' = max(ln_y_`i') if date > 2000
	}

forvalues i = 1(1)5 {
	gen mu`i' = (0.8*2*_pi)/(b`i' - a`i')
	gen q`i'  = 0.2*_pi - (mu`i'*a`i') + mu`i'*ln_y_`i'
	} 

drop a1-b5
	
forvalues i = 1(1)5 {
	gen cos_q`i' = cos(q`i')
	gen sin_q`i' = sin(q`i')
	gen cos_q`i'`i' = cos(2*q`i')
	gen sin_q`i'`i' = sin(2*q`i')
	}


gen cos_q12 = cos(q1 + q2)
gen sin_q12 = sin(q1 + q2)
gen cos_q13 = cos(q1 + q3)
gen sin_q13 = sin(q1 + q3)
gen cos_q14 = cos(q1 + q4)
gen sin_q14 = sin(q1 + q4)
gen cos_q15 = cos(q1 + q5)
gen sin_q15 = sin(q1 + q5)
gen cos_q23 = cos(q2 + q3)
gen sin_q23 = sin(q2 + q3)
gen cos_q24 = cos(q2 + q4)
gen sin_q24 = sin(q2 + q4)
gen cos_q25 = cos(q2 + q5)
gen sin_q25 = sin(q2 + q5)
gen cos_q34 = cos(q3 + q4)
gen sin_q34 = sin(q3 + q4)
gen cos_q35 = cos(q3 + q5)
gen sin_q35 = sin(q3 + q5)
gen cos_q45 = cos(q4 + q5)
gen sin_q45 = sin(q4 + q5)

*******************************************************************
*** SAVING THE DATA
sort company date
* cd to put in relevant directory here
save "Cost_fn_variables_no_subsidy_2inputs.dta", replace
