*******************************************************************
* PROGRAMME: COST FUNCTION ESTIMATION
*******************************************************************
*** PROGRAMME STRUCTURE:
* 1: IMPORT DATA
* 2: REGRESSIONS OF THE TRANSLOG COST FUNCTION
* 3: COMPUTING SCALE ECONOMIES FOR EACH BANK
* 4: COMPUTING THE VARIANCE OF SCALE ECONOMIES, DELTA METHOD

*******************************************************************
*** 1: IMPORT DATA
clear
set more off
* cd to put in relevant directory here

use "Cost_fn_variables_no_subsidy_2inputs.dta"

sort company date 
xtset company_id date

set matsize 1000
set emptycells drop

*******************************************************************
*** DROPPING REPEAT BANKS
* Banco Espirito Santo SA is a subsidiary of Esprito Santo Financial Group
drop if company == "Banco Espirito Santo SA"
* Banco Santander (Brasil) S.A. is a subsidiary of Banco Santander in Brasil
drop if company == "Banco Santander (Brasil) S.A."

*** DROPPING BROKERS (i.e. banks classified as "Investment Banking and Brokerage" in Capital IQ. They do not do retail banking)
drop if company == "The Goldman Sachs Group, Inc."
drop if company == "Morgan Stanley"
drop if company == "Nomura Holdings Inc."
drop if company == "Daiwa Securities Group Inc."
drop if company == "Macquarie Group, Ltd."
drop if company == "ICAP plc"
drop if company == "Mediobanca S.p.A." 
drop if company == "Queensland Treasury Corp."
drop if company == "Charles Schwab Corp."

*******************************************************************
*** 2: REGRESSIONS OF THE COST FUNCTION 
*** CROSS-EQUATION CONSTRAINTS FOR S_p
constraint 1  [cons_ln_total_cost]cons_ln_w = [S_p]_cons
constraint 2  [cons_ln_total_cost]cons_ln_w_ln_w = [S_p]cons_ln_w
constraint 3  [cons_ln_total_cost]cons_y_1_ln_w = [S_p]ln_y_1
constraint 4  [cons_ln_total_cost]cons_y_2_ln_w = [S_p]ln_y_2
constraint 5  [cons_ln_total_cost]cons_y_3_ln_w = [S_p]ln_y_3
constraint 6  [cons_ln_total_cost]cons_y_4_ln_w = [S_p]ln_y_4
constraint 7  [cons_ln_total_cost]cons_y_5_ln_w = [S_p]ln_y_5
constraint 8  [cons_ln_total_cost]cons_ln_w_ln_k = [S_p]ln_k
constraint 9  [cons_ln_total_cost]cons_ln_w_ln_p = [S_p]ln_p
constraint 10 [cons_ln_total_cost]cons_ln_w_ln_n = [S_p]ln_n

*******************************************************************
*** 2b: ORGANISATION OF RELEVANT VARIABLES FOR THE COST FUNCTION 
*** All updated macro data. Bank sys, regional average
global macro_controls  	gdp_growth_real interest_rates gdp_pc credit_gdp_ratio bank_sys_concentration
global bank_fe  		i.company_id
global time_fe  		i.date

global lny 				ln_y_1-ln_y_5 
global lnylny  			H_ln_y_1_ln_y_1-cons_H_ln_y_4_ln_y_5 
global lnw  			cons_ln_w 
global lnwlnw  			cons_ln_w_ln_w
global lnk  			ln_k 
global lnn  			ln_n
global lnp				ln_p 
global lnklnk  			H_ln_k_ln_k 
global lnnlnn  			H_ln_n_ln_n 
global lnplnp  			H_ln_p_ln_p 
global lnklnn  			ln_k_ln_n 
global lnklnp  			ln_k_ln_p 
global lnplnn  			ln_p_ln_n
global lnylnw  			cons_y_1_ln_w-cons_y_5_ln_w 
global lnwlnk  			cons_ln_w_ln_k
global lnwlnn  			cons_ln_w_ln_n
global lnwlnp  			cons_ln_w_ln_p
global lnylnk  			ln_y_1_ln_k-ln_y_5_ln_k 
global lnylnn  			ln_y_1_ln_n-ln_y_5_ln_n 
global lnylnp  			ln_y_1_ln_p-ln_y_5_ln_p 

global trig_terms  		cos_q1-sin_q555

 

*******************************************************************
*** 2c: REGRESSIONS OF THE COST FUNCTION 
*** TRANSLOG SPECIFICATION
*** NO MODELLING OF RISK
*** MACRO CONTROLS, NO TIME DUMMIES 
*local translog 			cons_ln_total_cost $lny $lnylny $lnylnw $lnylnk $lnylnp $lnw $lnwlnw $lnk $lnklnk $lnwlnk $lnp $lnplnp $lnwlnp $lnklnp $bank_fe $macro_controls $time_fe 
local translog 			cons_ln_total_cost $lny $lnylny $lnylnw $lnylnk $lnylnp $lnw $lnwlnw $lnk $lnklnk $lnwlnk $lnp $lnplnp $lnwlnp $lnklnp $bank_fe $macro_controls

* MAIN RESULTS
sureg (`translog') (S_p cons_ln_w ln_y_1-ln_y_5 ln_k ln_p) if date > 2000, c(1-9)
estimates store SUR
sureg (`translog') (S_p cons_ln_w ln_y_1-ln_y_5 ln_k ln_p) if date > 2000 & implicit_subsidy == 0, c(1-9)
estimates store SUR_ns

*******************************************************************
* 3: SCALE ECONOMIES CALCULATIONS

sort company date 
xtset company_id date

set matsize 550
set emptycells drop

*******************************************************************
*** 3a: COMPUTING SCALE ECONOMIES FOR EACH BANK
*** TRANSLOG SPECIFICATION
*** NO MODELLING OF RISK
sort company date 
xtset company_id date
foreach model of newlist SUR_ns {
est restore `model'
bysort company date: gen a = _b[ln_y_1] + _b[ln_y_2] + _b[ln_y_3] + _b[ln_y_4] + _b[ln_y_5]
bysort company date: gen b_y1 = ln_y_1 * (_b[H_ln_y_1_ln_y_1] + _b[cons_H_ln_y_1_ln_y_2] + _b[cons_H_ln_y_1_ln_y_3] + _b[cons_H_ln_y_1_ln_y_4] + _b[cons_H_ln_y_1_ln_y_5])
bysort company date: gen b_y2 = ln_y_2 * (_b[H_ln_y_2_ln_y_2] + _b[cons_H_ln_y_1_ln_y_2] + _b[cons_H_ln_y_2_ln_y_3] + _b[cons_H_ln_y_2_ln_y_4] + _b[cons_H_ln_y_2_ln_y_5])
bysort company date: gen b_y3 = ln_y_3 * (_b[H_ln_y_3_ln_y_3] + _b[cons_H_ln_y_1_ln_y_3] + _b[cons_H_ln_y_2_ln_y_3] + _b[cons_H_ln_y_3_ln_y_4] + _b[cons_H_ln_y_3_ln_y_5])
bysort company date: gen b_y4 = ln_y_4 * (_b[H_ln_y_4_ln_y_4] + _b[cons_H_ln_y_1_ln_y_4] + _b[cons_H_ln_y_2_ln_y_4] + _b[cons_H_ln_y_3_ln_y_4] + _b[cons_H_ln_y_4_ln_y_5])
bysort company date: gen b_y5 = ln_y_5 * (_b[H_ln_y_5_ln_y_5] + _b[cons_H_ln_y_1_ln_y_5] + _b[cons_H_ln_y_2_ln_y_5] + _b[cons_H_ln_y_3_ln_y_5] + _b[cons_H_ln_y_4_ln_y_5])
bysort company date: gen b = b_y1 + b_y2 + b_y3 + b_y4 + b_y5
bysort company date: gen c = cons_ln_w * (_b[cons_y_1_ln_w] + _b[cons_y_2_ln_w] + _b[cons_y_3_ln_w] + _b[cons_y_4_ln_w] + _b[cons_y_5_ln_w]) 
bysort company date: gen d = ln_k * (_b[ln_y_1_ln_k] + _b[ln_y_2_ln_k] + _b[ln_y_3_ln_k] + _b[ln_y_4_ln_k] + _b[ln_y_5_ln_k])
bysort company date: gen g = ln_p * (_b[ln_y_1_ln_p] + _b[ln_y_2_ln_p] + _b[ln_y_3_ln_p] + _b[ln_y_4_ln_p] + _b[ln_y_5_ln_p])
bysort company date: gen cf_`model' = 1/(a + b + c + d + g)
bysort company date: gen dlnCdlnk_`model' = _b[ln_k] + ln_k*_b[H_ln_k_ln_k] +  cons_ln_w*_b[cons_ln_w_ln_k] + ln_y_1*_b[ln_y_1_ln_k] + ln_y_2*_b[ln_y_2_ln_k] + ln_y_3*_b[ln_y_3_ln_k] + ln_y_4*_b[ln_y_4_ln_k] + ln_y_5*_b[ln_y_5_ln_k] + ln_p*_b[ln_k_ln_p]
bysort company date: gen ec_`model' = (1-dlnCdlnk_`model')/(a + b + c + d + g)
drop  a-g
}


*******************************************************************
* 4: COMPUTING THE VARIANCE OF SCALE ECONOMIES, DELTA METHOD

*******************************************************************
*** 2a: COMPUTING VARIANCE OF SCALE ECONOMIES FOR EACH BANK 
*** NO RISK TAKING
*** TRANSLOG SPECIFICATION
foreach model of newlist SUR_ns {
est restore `model'
matrix V_`model'  = e(V)
matselrc V_`model' Vb_`model', r(1/35) c(1/35)

sort company date 
xtset company_id date
est restore `model'
bysort company date: gen d_a1  = -(cf_`model')^2
bysort company date: gen d_a2  = -(cf_`model')^2
bysort company date: gen d_a3  = -(cf_`model')^2
bysort company date: gen d_a4  = -(cf_`model')^2
bysort company date: gen d_a5  = -(cf_`model')^2
bysort company date: gen d_a11 = -ln_y_1*(cf_`model')^2
bysort company date: gen d_a22 = -ln_y_2*(cf_`model')^2
bysort company date: gen d_a33 = -ln_y_3*(cf_`model')^2
bysort company date: gen d_a44 = -ln_y_4*(cf_`model')^2
bysort company date: gen d_a55 = -ln_y_5*(cf_`model')^2
bysort company date: gen d_a12 = -(ln_y_1+ln_y_2)*(cf_`model')^2
bysort company date: gen d_a13 = -(ln_y_1+ln_y_3)*(cf_`model')^2
bysort company date: gen d_a14 = -(ln_y_1+ln_y_4)*(cf_`model')^2
bysort company date: gen d_a15 = -(ln_y_1+ln_y_5)*(cf_`model')^2
bysort company date: gen d_a23 = -(ln_y_2+ln_y_3)*(cf_`model')^2
bysort company date: gen d_a24 = -(ln_y_2+ln_y_4)*(cf_`model')^2
bysort company date: gen d_a25 = -(ln_y_2+ln_y_5)*(cf_`model')^2
bysort company date: gen d_a34 = -(ln_y_3+ln_y_4)*(cf_`model')^2
bysort company date: gen d_a35 = -(ln_y_3+ln_y_5)*(cf_`model')^2
bysort company date: gen d_a45 = -(ln_y_4+ln_y_5)*(cf_`model')^2
bysort company date: gen d_n1  = -cons_ln_w*(cf_`model')^2
bysort company date: gen d_n2  = -cons_ln_w*(cf_`model')^2
bysort company date: gen d_n3  = -cons_ln_w*(cf_`model')^2
bysort company date: gen d_n4  = -cons_ln_w*(cf_`model')^2
bysort company date: gen d_n5  = -cons_ln_w*(cf_`model')^2
bysort company date: gen d_p11 = -ln_k*(cf_`model')^2
bysort company date: gen d_p12 = -ln_k*(cf_`model')^2
bysort company date: gen d_p13 = -ln_k*(cf_`model')^2
bysort company date: gen d_p14 = -ln_k*(cf_`model')^2
bysort company date: gen d_p15 = -ln_k*(cf_`model')^2
bysort company date: gen d_p21 = -ln_p*(cf_`model')^2
bysort company date: gen d_p22 = -ln_p*(cf_`model')^2
bysort company date: gen d_p23 = -ln_p*(cf_`model')^2
bysort company date: gen d_p24 = -ln_p*(cf_`model')^2
bysort company date: gen d_p25 = -ln_p*(cf_`model')^2

sort assetbuckets
foreach var of varlist d_a1-d_p25 {
egen sum00_`var' = sum(`var') if _est_`model' == 1
egen sum11_`var' = sum(`var') if assetbuckets == "1 <$100bn" & _est_`model' == 1
egen sum22_`var' = sum(`var') if assetbuckets == "2 $100bn-$500bn" & _est_`model' == 1
egen sum33_`var' = sum(`var') if assetbuckets == "3 $500bn-$1tn" & _est_`model' == 1
egen sum44_`var' = sum(`var') if assetbuckets == "4 $1tn-$2tn" & _est_`model' == 1
egen sum55_`var' = sum(`var') if assetbuckets == "5 >$2tn" & _est_`model' == 1
}

foreach var of varlist d_a1-d_p25 {
egen sum0_`var' = max(sum00_`var')
}
foreach var of varlist d_a1-d_p25 {
egen sum1_`var' = max(sum11_`var')
}
foreach var of varlist d_a1-d_p25 {
egen sum2_`var' = max(sum22_`var')
}
foreach var of varlist d_a1-d_p25 {
egen sum3_`var' = max(sum33_`var')
}
foreach var of varlist d_a1-d_p25 {
egen sum4_`var' = max(sum44_`var')
}
foreach var of varlist d_a1-d_p25 {
egen sum5_`var' = max(sum55_`var')
}
foreach i of numlist 0/5 {
egen count`i' = count(sum`i'`i'_d_p25) 
}

sort company date
foreach i of numlist 0/5 {
capture mkmat sum`i'_d_a1-sum`i'_d_p25 if company_id == 1 & date == 2001, matrix(gB_`model'_`i')
capture matrix var_`model'_`i' = gB_`model'_`i'*Vb_`model'*gB_`model'_`i''
capture svmat var_`model'_`i', names(temp)
capture replace temp1 = sum(temp)
capture gen se_`model'_`i' = (1/(count`i')^2)*temp1
capture replace se_`model'_`i' = sqrt(se_`model'_`i')
capture drop temp1
}

bysort company date: gen se_mean_cf_`model' = .
capture replace se_mean_cf_`model' = se_`model'_1 if assetbuckets == "1 <$100bn"
capture replace se_mean_cf_`model' = se_`model'_2 if assetbuckets == "2 $100bn-$500bn"
capture replace se_mean_cf_`model' = se_`model'_3 if assetbuckets == "3 $500bn-$1tn"
capture replace se_mean_cf_`model' = se_`model'_4 if assetbuckets == "4 $1tn-$2tn"
capture replace se_mean_cf_`model' = se_`model'_5 if assetbuckets == "5 >$2tn"

drop d_a1-d_p25 sum00_d_a1-count5
* END LOOP
}


*** SUMMARY STATISTICS OF SCALE ECONOMIES ESTIMATES -- no TBTF in prodn technology
foreach model of newlist SUR {
table assetbuckets if date > 2000 & _est_`model' == 1, c(count cf_`model'_ns mean cf_`model'_ns mean se_mean_cf_`model'_ns)
}
foreach model of newlist SUR {
table _est_`model' if date > 2000 & _est_`model'==1, c(count cf_`model'_ns mean cf_`model'_ns mean se_`model'_ns_0)
}


foreach model of newlist SUR {
table assetbuckets implicit_subsidy if date > 2000 & _est_`model' == 1, c(count cf_`model'_ns mean cf_`model'_ns mean se_mean_cf_`model'_ns)
}
foreach model of newlist SUR {
table _est_`model' implicit_subsidy if _est_`model'==1, c(count cf_`model'_ns mean cf_`model'_ns mean se_`model'_ns_0)
}

sort order
*save "Scale_economies_2input_subsidy.dta"
